dzldockstack dzltabstrip {
  background-color: @theme_bg_color;
}

dzldockstack dzltabstrip button.control:hover:checked,
dzldockstack dzltabstrip button.control:checked,
dzldockstack dzltabstrip dzltab:hover:checked,
dzldockstack dzltabstrip dzltab:checked {
  color: @theme_fg_color;
  background-color: @theme_base_color;
  border-color: @borders;
}

dzldockstack dzltabstrip dzltab {
  font-size: 0.9em;
  margin-top: 3px;
  padding-bottom: 3px;
  border-style: solid;
  border-color: transparent;
  border-right-width: 1px;
  border-left-width: 1px;
  border-top-width: 1px;
  border-bottom-width: 0px;
}

dzldockstack dzltabstrip dzltab:backdrop:checked,
dzldockstack dzltabstrip dzltab:backdrop {
  background-color: alpha(@theme_base_color, 0.25);
}

dzldockstack dzltabstrip dzltab:checked {
  border-left-color: @borders;
  border-right-color: @borders;
  border-top-color: @borders;
}

dzldockstack dzltabstrip dzltab:checked:first-child {
  border-left-color: alpha(@theme_base_color, 0.25);
}

dzldockstack dzltabstrip dzltab:checked:last-child {
  border-right-color: alpha(@theme_base_color, 0.25);
}
